/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-ID has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moaspss.util;

import java.io.IOException;
import java.io.InputStream;
import java.util.Map;

import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * An <code>EntityResolver</code> that maps system IDs to 
 * <code>InputStream</code>s.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class StreamEntityResolver implements EntityResolver {
  
  /** A mapping from Public ID or System ID to an <code>InputStream</code>
   * containing the entity. */
  private Map mappedEntities;
  
  /**
   * Create a <code>StreamEntityResolver</code>.
   * 
   * @param mappedEntities A mapping from public or system IDs 
   * (<code>String</code> objects) to <code>InputStream</code>s. 
   */
  public StreamEntityResolver(Map mappedEntities) {
    this.mappedEntities = mappedEntities;
  }

  /**
   * Resolve an entity by looking it up in the mapped entities.
   * 
   * First, the public ID is looked up in the mapping, then the system ID.
   * 
   * @param publicId The public ID of the entity.
   * @param systemId The system ID of the entity.
   * @return An <code>InputStream</code> containing the entity or 
   * <code>null</code> if no entity could be found. 
   * @throws SAXException Signalling a parsing exception.
   * @throws IOException Error reading the entity.
   */
  public InputSource resolveEntity(String publicId, String systemId) 
    throws SAXException, IOException {
      
    InputSource src = null;
      
    if (publicId != null && mappedEntities.get(publicId) != null) {
      src = new InputSource((InputStream) mappedEntities.get(publicId));
    } else if (systemId != null && mappedEntities.get(systemId) != null) {
      src = new InputSource((InputStream) mappedEntities.get(systemId));
    }
    
    if (src != null) {
      src.setPublicId(publicId);
      src.setSystemId(systemId);
    }
    
    return src;
  }
}
