/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.invoke;

import java.util.ArrayList;
import java.util.List;

import at.gv.egovernment.moa.spss.api.SPSSFactory;
import at.gv.egovernment.moa.spss.api.cmssign.CMSSignatureResponse;
import at.gv.egovernment.moa.spss.api.cmssign.CreateCMSSignatureResponse;
import at.gv.egovernment.moa.spss.api.xmlsign.ErrorResponse;

/**
 * A class to build a <code>CreateCMSSignatureResponse</code>.
 * 
 * <p>The methods <code>addSignature()</code> and <code>addError()</code> may be
 * called in any combination to add <code>CMSignature</code> and
 * <code>ErrorResponse</code> elements to the response. One of these functions
 * must be called at least once to produce a 
 * <code>CreateCMSSignatureResponse</code>.</p>
 * 
 * <p>The <code>getResponseElement()</code> method then returns the
 * <code>CreateXMLSignatureResponse</code> built so far.</p>
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class CreateCMSSignatureResponseBuilder {

  /** The <code>SPSSFactory</code> for creating API objects. */
  private SPSSFactory factory = SPSSFactory.getInstance();
  /** The elements to add to the response. */
  private List responseElements = new ArrayList();

  /**
   * Get the <code>CreateCMSSignatureResponse</code> built so far.
   * 
   * @return The <code>CreateCMSSignatureResponse</code> built so far.
   */
  public CreateCMSSignatureResponse getResponse() {
    return factory.createCreateCMSSignatureResponse(responseElements);
  }

  /**
   * Add a <code>SignatureEnvironment</code> element to the response.
   * 
   * @param signatureEnvironment The content to put under the
   * <code>SignatureEnvironment</code> element. This should either be a
   * <code>dsig:Signature</code> element (in case of a detached signature) or
   * the signature environment containing the signature (in case of
   * an enveloping signature).
   */
  public void addCMSSignature(String base64value) {
    CMSSignatureResponse responseElement = 
      factory.createCMSSignatureResponse(base64value);
    responseElements.add(responseElement);
  }

  /**
   * Add a <code>ErrorResponse</code> element to the response.
   *  
   * @param errorCode The error code.
   * @param info Additional information about the error.
   */
  public void addError(String errorCode, String info) {
    ErrorResponse errorResponse = 
      factory.createErrorResponse(Integer.parseInt(errorCode), info);
    responseElements.add(errorResponse);
  }

}
