/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.invoke;

import iaik.server.modules.cmsverify.CMSSignatureVerificationResult;
import iaik.server.modules.pdfverify.PDFSignatureVerificationResult;
import iaik.server.cmspdfverify.CertificateValidationResult;

import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;

import at.gv.egovernment.moa.spss.MOAException;
import at.gv.egovernment.moa.spss.api.SPSSFactory;
import at.gv.egovernment.moa.spss.api.cmsverify.VerifyCMSSignatureResponse;
import at.gv.egovernment.moa.spss.api.cmsverify.VerifyCMSSignatureResponseElement;
import at.gv.egovernment.moa.spss.api.common.CheckResult;
import at.gv.egovernment.moa.spss.api.common.SignerInfo;
import at.gv.egovernment.moa.spss.server.config.TrustProfile;

/**
 * A class to build a <code>VerifyCMSSignatureResponse</code> object.
 * 
 * <p>Via subsequent calls to <code>addResult()</code> a number of results from
 * a CMS signature verification can be added to the response.</p>
 * 
 * <p>The <code>getResponseElement()</code> method then returns the
 * <code>VerifyCMSSignatureResponse</code> built so far.</p>
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class VerifyCMSSignatureResponseBuilder {
  /** The <code>SPSSFactory</code> for creating API objects. */
  private SPSSFactory factory = SPSSFactory.getInstance();
  /** The elements making up the response. */
  private List responseElements = new ArrayList();

  /**
   * Get the <code>VerifyCMSSignatureResponse</code> built so far.
   * 
   * @return The <code>VerifyCMSSignatureResponse</code> built so far.
   */
  public VerifyCMSSignatureResponse getResponse() {
    return factory.createVerifyCMSSignatureResponse(responseElements);
  }

  /**
   * Add a verification result to the response.
   * 
   * @param result The result to add.
   * @param trustprofile The actual trustprofile
   * @param checkQCFromTSL <code>true</code>, if the TSL check verifies the 
   * 		certificate as qualified, otherwise <code>false</code>.
   * @param checkSSCD <code>true</code>, if the TSL check verifies the 
   * 		signature based on a SSDC, otherwise <code>false</code>.
   * @param sscdSourceTSL <code>true</code>, if the SSCD information comes from the TSL, 
   * 		otherwise <code>false</code>.
 * @throws MOAException 
   */
  public void addResult(CMSSignatureVerificationResult result, TrustProfile trustProfile, boolean checkQC, boolean qcSourceTSL, boolean checkSSCD, boolean sscdSourceTSL, String issuerCountryCode, List adesResults)
    throws MOAException {
	  
    CertificateValidationResult certResult =
      result.getCertificateValidationResult();
    int signatureCheckCode =
      result.getSignatureValueVerificationCode().intValue();
    int certificateCheckCode = certResult.getValidationResultCode().intValue();
         
    VerifyCMSSignatureResponseElement responseElement;
    SignerInfo signerInfo;
    CheckResult signatureCheck;
    CheckResult certificateCheck;

    boolean qualifiedCertificate = checkQC;
    
    // add SignerInfo element
    signerInfo =
      factory.createSignerInfo(
        (X509Certificate) certResult.getCertificateChain().get(0),
        qualifiedCertificate,
        qcSourceTSL,
        certResult.isPublicAuthorityCertificate(),
        certResult.getPublicAuthorityID(),
        checkSSCD,
        sscdSourceTSL,
        issuerCountryCode);

    // add SignatureCheck element
    signatureCheck = factory.createCheckResult(signatureCheckCode, null);

    // add CertificateCheck element
    certificateCheck = factory.createCheckResult(certificateCheckCode, null);
    
    // build the response element
    responseElement =
      factory.createVerifyCMSSignatureResponseElement(
        signerInfo,
        signatureCheck,
        certificateCheck, 
        adesResults);
    responseElements.add(responseElement);
  }
  
  /**
   * Add a verification result to the response.
   * 
   * @param result The result to add.
   * @param trustprofile The actual trustprofile
   * @param checkQCFromTSL <code>true</code>, if the TSL check verifies the 
   * 		certificate as qualified, otherwise <code>false</code>.
   * @param checkSSCD <code>true</code>, if the TSL check verifies the 
   * 		signature based on a SSDC, otherwise <code>false</code>.
   * @param sscdSourceTSL <code>true</code>, if the SSCD information comes from the TSL, 
   * 		otherwise <code>false</code>.
 * @throws MOAException 
   */
  public void addResult(PDFSignatureVerificationResult result, TrustProfile trustProfile, boolean checkQC, boolean qcSourceTSL, boolean checkSSCD, boolean sscdSourceTSL, String issuerCountryCode, List adesResults)
    throws MOAException {
	  
    CertificateValidationResult certResult =
      result.getCertificateValidationResult();
    int signatureCheckCode =
      result.getSignatureValueVerificationCode().intValue();
    int certificateCheckCode = certResult.getValidationResultCode().intValue();
         
    VerifyCMSSignatureResponseElement responseElement;
    SignerInfo signerInfo;
    CheckResult signatureCheck;
    CheckResult certificateCheck;

    boolean qualifiedCertificate = checkQC;
    
    // add SignerInfo element
    signerInfo =
      factory.createSignerInfo(
        (X509Certificate) certResult.getCertificateChain().get(0),
        qualifiedCertificate,
        qcSourceTSL,
        certResult.isPublicAuthorityCertificate(),
        certResult.getPublicAuthorityID(),
        checkSSCD,
        sscdSourceTSL,
        issuerCountryCode);

    // add SignatureCheck element
    signatureCheck = factory.createCheckResult(signatureCheckCode, null);

    // add CertificateCheck element
    certificateCheck = factory.createCheckResult(certificateCheckCode, null);
    
    // build the response element
    responseElement =
      factory.createVerifyCMSSignatureResponseElement(
        signerInfo,
        signatureCheck,
        certificateCheck, 
        adesResults);
    responseElements.add(responseElement);
  }

}
