/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.service;

import iaik.pki.revocation.RevocationSourceTypes;
import iaik.pki.store.revocation.archive.Archive;
import iaik.pki.store.revocation.archive.ArchiveFactory;

import java.util.Date;

import at.gv.egovernment.moa.spss.server.config.ConfigurationProvider;
import at.gv.egovernment.moa.spss.server.logging.TransactionId;
import at.gv.egovernment.moa.spss.util.MessageProvider;
import at.gv.egovernment.moaspss.logging.LogMsg;
import at.gv.egovernment.moaspss.logging.Logger;

/**
 * A <code>Runnable</code> for periodically cleaning up the revocation archive. 
 * @author Patrick Peck
 * @version $Id$
 */
public class RevocationArchiveCleaner implements Runnable {

  /** The inverval between two clean-ups of the revocation archive. */
  private long archiveCleanupInterval;

  /**
   * Create a new <code>RevocationArchiveCleaner</code>.
   * 
   * @param archiveCleanupInterval The interval between two clean-ups of the
   * revocation archive.
   */
  public RevocationArchiveCleaner(long archiveCleanupInterval) {
    this.archiveCleanupInterval = archiveCleanupInterval;
  }

  /**
   * Run the <code>RevocationArchiveCleaner</code> in its own 
   * <code>Thread</code>.
   */
  public void run() {
    while (true) {
      try {
        ConfigurationProvider config = ConfigurationProvider.getInstance();
        boolean enableArchiving = config.getEnableRevocationArchiving();

        if (enableArchiving) 
        {
          Archive archive = ArchiveFactory.getInstance().getArchive();
          long archiveDurationMillis =
            (long) config.getCRLArchiveDuration() * 86400000;

          // delete old archive data
          if (archiveDurationMillis > 0) {
            Date olderThan =
              new Date(System.currentTimeMillis() - archiveDurationMillis);

            archive.deleteOldArchiveEntries(
              RevocationSourceTypes.CRL,
              olderThan,
              new TransactionId("RevocationArchiveCleaner"));
          }
        }

      } catch (Exception e) {
        MessageProvider msg = MessageProvider.getInstance();
        Logger.error(new LogMsg(msg.getMessage("init.02", null)), e);
      }

      // sleep 
      try {
        Thread.sleep(archiveCleanupInterval * 1000);
      } catch (InterruptedException e) {
        // ok to do nothing here
      }

    }
  }

}
