/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.transaction;

/**
 * Provides each thread with an instance of <code>TransactionContext</code>.
 * 
 * The single instance of the <code>TransactionContextManager</code> should be
 * used to access contextual information for each web service transaction, e.g.
 * the transaction ID, MOA configuration, client certificate, etc.
 * 
 * @author Stefan Knirsch
 * @author Patrick Peck
 */
public class TransactionContextManager {

  /** The single instance of <code>TransactionContextManager</code> */ 
  private static TransactionContextManager instance = null;
  
  /** Contains a single <code>TransactionContext</code> for each thread. */
  private ThreadLocal context = null;

  /**
   * Get the single instance of <code>TransactionContextManager</code>.
   *   
   * @return The single instanc of <code>TransactionContextManager</code>.
   */
  public static synchronized TransactionContextManager getInstance() {
    if (instance == null) {
      instance = new TransactionContextManager();
    }
    return instance;
  }

  /**
   * Creates a new <code>TransactionContextManager</code>.
   * 
   * Protected to disallow direct instantiation.
   */
  protected TransactionContextManager() {
    context = new ThreadLocal();
  }

  /**
   * Set the <code>TransactionContext</code> for the current thread.
   * 
   * @param txContext The <code>TransactionContext</code> for this thread.
   */
  public void setTransactionContext(TransactionContext txContext) {
    context.set(txContext);
  }

  /**
   * Get the <code>TransactionContext</code> for the current thread.
   * 
   * @return The <code>TransactionContext</code> for the current thread or
   * <code>null</code>, if none has been set (or if this method is being invoked
   * outside the bounds of a transaction).
   */
  public TransactionContext getTransactionContext() {
    return (TransactionContext) context.get();
  }

}
