/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.transaction;


/**
 * A generator for unique transaction IDs.
 * 
 * <p>The transaction IDs are of the form "<base>-<counter>", where:
 * <ul> 
 * <li><code>base</code> is initialized with the system time when this class is
 * being loaded</li>
 * <li><code>counter</code> is incremented sequentially on each call to
 * <code>nextID()</code></li>
 * </ul>
 * </p>
 * 
 * <p> Assuming that it is highly unlikely that MOA servers are started at
 * exactly the same time instant, the mechanism provided by this class should
 * guarantee unique transaction IDs across multiple restarts and/or instances of
 * the server.</p>
 * 
 * @author Patrick Peck
 * @author Stefan Knirsch
 */
public class TransactionIDGenerator {

  /** Request sequence number. */
  private static long counter = 0;
  /** The base value to which to append the sequence number. */
  private static String base = null;

  /**
   * Set up the initial base value.
   */
  static {
    synchronized (TransactionIDGenerator.class) {
      base = Long.toString(System.currentTimeMillis());
    }
  }

  /**
   * Returns the next transaction ID.
   * 
   * @return The next transaction ID.
   */
  public static synchronized String nextID() {
    counter++;

    return (base + "-" + counter);
  }

}
